{*********************************}

  {Crditos-Autor}
  {* Criado por Vincius Magno *}
  {* E-mail magnolara@aol.com  *}

{*********************************}

unit MRegression;

interface

uses
  SysUtils, Classes, Forms, registry, Windows;


type
  TMRegression = class(TComponent)
  private
    { Private declarations }
    FAboutAutor: String;
    FAboutRelease: String;
    FMaxDay: integer;
    FMaxExe: integer;
    FNowDay: integer;
    FNowExe: integer;
    FUpDate: TDateTime;
    FInstallDate: TDateTime;
    FActive: boolean;
    FReg: string;
    FOnLimite: TNotifyEvent;
    FOnCracked: TNotifyEvent;
    FOnWelcome: TNotifyEvent;
    FOnReturnDate: TNotifyEvent;
    procedure SetActive(Value: Boolean);
    procedure SetNowDay(Value: integer);
    procedure SetNowExe(Value: integer);
    procedure SetAboutAutor(Value: string);
    procedure SetAboutRelease(Value: string);
    procedure SetInstallDate(Value: TDateTime);
    procedure SetUpDate(Value: TDateTime);
    procedure CheckLock;
    procedure WriteRegistry(S: string);
    function ReadRegistry: string;
    function IsLock: Boolean;
  protected
    { Protected declarations }
    procedure Loaded; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property Active: Boolean read FActive write SetActive;
    property Registro: String read FReg write FReg;
    property UpDate: TDateTime read FUpDate write SetUpDate;
    property InstallDate: TDateTime read FInstallDate write SetInstallDate;
    property MaxDay: integer read FMaxDay write FMaxDay;
    property MaxExe: integer read FMaxExe write FMaxExe;
    property NowDay: integer read FNowDay write SetNowDay;
    property NowExe: integer read FNowExe write SetNowExe;
    property AboutAutor: String read FAboutAutor write SetAboutAutor;
    property AboutRelease: String read FAboutRelease write SetAboutRelease;
    // Eventos
    property OnLimite: TNotifyEvent read FOnLimite write FOnLimite;
    property OnCracked: TNotifyEvent read FOnCracked write FOnCracked;
    property OnWelcome: TNotifyEvent read FOnWelcome write FOnWelcome;
    property OnReturnDate: TNotifyEvent read FOnReturnDate write FOnReturnDate;
  end;

procedure Register;

implementation
const
RegPath = '\Software\Empresa\Produto\';

Function Encode(x:String): String;
var
  s : string[255];
  c : array[0..255] of Byte absolute s;
  i : integer;
begin
  s := x;
  for i := 1 to ord(s[0]) do
  begin
    c[i] := 23 xor c[i];
  end;
  Result := s;
end;

Function Decode(x:String): String;
var
  i : integer;
  s : string[255];
  c : array[0..255] of Byte absolute s;
begin
  s := x;
  for i := 1 to Length(s) do
  begin
    s[i] := char(23 xor ord(c[i]));
  end;
  Result := s;
end;

constructor TMRegression.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FActive := True;
  FMaxDay := 15;
  FMaxExe := 100;
  FNowDay := 0;
  FNowExe := 0;
  FUpDate := Date;
  FInstallDate := Date;
  FReg := RegPath;
  FAboutAutor := 'Vincius Magno Lara - magnolara@aol.com';
  FAboutRelease := '1.0 - 02/05/2003';
end;

procedure TMRegression.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) and FActive then
    CheckLock;
end;

function TMRegression.ReadRegistry: string;
var
  Reg: TRegistry;
begin
  Result := '';
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    if Reg.OpenKey(FReg, False) then
      Result := Reg.ReadString('RegressionLock');
  finally
    Reg.CloseKey;
    Reg.Free;
  end;
end;

procedure TMRegression.WriteRegistry(S: string);
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    if Reg.OpenKey(FReg, True) then
      Reg.WriteString('RegressionLock', S);
  finally
    Reg.CloseKey;
    Reg.Free;
  end;
end;

function TMRegression.IsLock: Boolean;
begin
  if (FNowDay >= FMaxDay) or
     (FNowExe >= FMaxExe) then
    result := True
  else result := False;
end;

procedure TMRegression.CheckLock;
var
  s: string;
begin
  s := Decode(ReadRegistry);

  // OnWelcome
  if (s = '') and
     (FUpDate = Date) then
  begin
    FNowDay := FNowDay + 1;
    FNowExe := FNowExe;
    FUpDate := Date;
    WriteRegistry(Encode(Copy(DateToStr(FUpDate),1,2) +
                         Copy(DateToStr(FUpDate),4,2) +
                         Copy(DateToStr(FUpDate),7,4) + '.' +
                         FormatFloat('0000',FNowDay) + '.' +
                         FormatFloat('0000',FNowExe)
                        ));
    s := Decode(ReadRegistry);
    if Assigned(FOnWelcome) then
      FOnWelcome(Self);
  end
  // OnCracked
  else if (s = '') and
          (FUpDate <> Date) then
  begin
    if Assigned(FOnCracked) then
      FOnCracked(Self);
  end;

  // OnCracked
  if (Length(s) > 0) and
     (Length(s) <> 18)then
  begin
    if Assigned(FOnCracked) then
      FOnCracked(Self);
  end;

  // (Se no for vazio, lana as informaes)
  FUpDate := StrToDate(Copy(s,1,2) + '/' +
                       Copy(s,3,2) + '/' +
                       Copy(s,5,4));
  FNowDay := StrToInt(Copy(s,10,4));
  FNowExe := StrToInt(Copy(s,15,4));

  // (Novo dia)
  if FUpDate < Date then
  begin
    FUpDate := Date;
    FNowDay := StrToInt(Copy(s,10,4)) + 1;
    FNowExe := StrToInt(Copy(s,15,4)) + 1;
    WriteRegistry(Encode(Copy(DateToStr(FUpDate),1,2) +
                         Copy(DateToStr(FUpDate),4,2) +
                         Copy(DateToStr(FUpDate),7,4) + '.' +
                         FormatFloat('0000',FNowDay) + '.' +
                         FormatFloat('0000',FNowExe)
                         ));

    // (V se este novo dia no fez execeder o limite)
    if IsLock = True then
      if Assigned(FOnLimite) then
        FOnLimite(Self)
  end
  // OnReturnDate (Usurio retornou data do windows)
  else if Date < FUpDate then
  begin
    if Assigned(FOnReturnDate) then
      FOnReturnDate(Self);
  end
  else begin
    // (No  um novo dia, mas vamos lanar a execuo)
    FUpDate := Date;
    FNowDay := StrToInt(Copy(s,10,4));
    FNowExe := StrToInt(Copy(s,15,4)) + 1;
    WriteRegistry(Encode(Copy(DateToStr(FUpDate),1,2) +
                         Copy(DateToStr(FUpDate),4,2) +
                         Copy(DateToStr(FUpDate),7,4) + '.' +
                         FormatFloat('0000',FNowDay) + '.' +
                         FormatFloat('0000',FNowExe)
                         ));

    // (Verifica se este novo lanamento no fez execeder o limite)
    if IsLock = True then
      if Assigned(FOnLimite) then
        FOnLimite(Self)
  end;
end;

procedure TMRegression.SetActive(Value: boolean);
begin
  if FActive <> Value then
  begin
    FActive := Value;
    if not (csDesigning in ComponentState) and
       not (csLoading in ComponentState) and FActive then
      CheckLock;
  end;
end;

procedure TMRegression.SetNowDay(Value: integer);
begin
  if FNowDay <> Value then
    FNowDay := Value;
end;

procedure TMRegression.SetNowExe(Value: integer);
begin
  if FNowExe <> Value then
    FNowExe := Value;
end;

procedure TMRegression.SetAboutAutor(Value: string);
begin
  ;
end;

procedure TMRegression.SetAboutRelease(Value: string);
begin
  ;
end;

procedure TMRegression.SetInstallDate(Value: TDateTime);
begin
  if FInstallDate <> Value then
    FInstallDate := Value;
end;

procedure TMRegression.SetUpDate(Value: TDateTime);
begin
  if FUpDate <> Value then
    FUpDate := Value;
end;

procedure Register;
begin
  RegisterComponents('MPack 2', [TMRegression]);
end;

end.
